@@ -1,9 +1,9 @@
-require 'rails-dom-testing'
+require "rails-dom-testing"
 
 module ActionDispatch
   module Assertions
-    autoload :ResponseAssertions, 'action_dispatch/testing/assertions/response'
-    autoload :RoutingAssertions, 'action_dispatch/testing/assertions/routing'
+    autoload :ResponseAssertions, "action_dispatch/testing/assertions/response"
+    autoload :RoutingAssertions, "action_dispatch/testing/assertions/routing"
 
     extend ActiveSupport::Concern
 
@@ -12,7 +12,7 @@ module ActionDispatch
     include Rails::Dom::Testing::Assertions
 
     def html_document
-      @html_document ||= if @response.content_type === Mime::XML
+      @html_document ||= if @response.content_type.to_s.end_with?("xml")
         Nokogiri::XML::Document.parse(@response.body)
       else
         Nokogiri::HTML::Document.parse(@response.body)
